{{
    This object acts as a wrapper for all I/O on the Bread Board.  Switch's, LED's
    Trimmers, etc.  This object should replace the PC_IO module when lap-top is
    unplugged. The Data method that was used to send info into cells in the
    spreadsheet, is used to set the led values in this object.  Sense there is no
    other output available (at present) on the breadboard.
    
    by ... Tapperman
    ver 1.0  3/11/2010   Initial version, borrowed from PC_IO.Spin
        1.1  3/27/2010   Added clock routines for DS1302 real time clock
        1.2  5/04/2010   Added LC routines, and a mode variable to control
        1.3  5/15/2010   Added 'filter' to quiet input.  Crude, limit bits.

    a comment about my comments (ha-ha) --> when you see comments that refer to
    BB (ie, mntd on BB), this should be interpreted as "Mounted on Bread Board"

}}
CON
  _CLKMODE = XTAL1 + PLL16X
  _XINFREQ = 5_000_000

   Ignr_bit  = 7                        ' how many bits to reject, low order
   RCPin     = 24                       ' pin to test RC with
   User2Pin  = 25                       ' user2 stop pin
   LCMax     = 47_500                   ' Use RC_Calibrate.Spin to get this value.
   RCMax     = 150_000                  ' Use RC_Calibrate.Spin to get this value.

OBJ
  lc       : "lcTime.Spin"                              ' reads LC tank control
  rc       : "rctime.spin"                              ' reads RC mntd on BB
  leds     : "SevenSegmentnbuzr"                        ' write data to leds (numbers only)
  rtc      : "DS1302_full.Spin"                         ' clock chip driver

VAR
    Long SMax,SMin,Gain                                 ' slider values
    Byte Stopped,mode                                   ' slider in raw mode?
    byte hour, minute, second, day, month, year, dow    ' clock variables.

        {the year variable, YY instead of YYYY.  Ie, 2009 stores as 09}
    
PUB Stop                                   ' Procedure - Stop
  if (mode & 1) > 0 
    LC.Stop
  else
    RC.Stop
  if (mode & 4) > 0 
    leds.Stop

PUB Start(aValue) | i                                   ' Procedure - Start
  {The mode is enbeded in aValue, as follows:

    mode bits
    --8-4-2-1-
      | | | |
      | | | +----> Device Slider1 reads. (LC if set, RC if clear)
      | | +------> RTC used if set, ignored if clear.
      | +--------> LED object used if set, ignored if clear.
      +----------> Reserved. Raw returns if set, scaled if clear.
  }
  aValue -= (mode := (aValue & 15))                     ' remove mode from aValue passed
  if (mode & 4) > 0 
    leds.start
  dira[leds#buzzpin]~~
  Stopped := True
  i~                                                    ' clear center/flag
  if (mode & 1) > 0                                     ' if mode LC selected 
    if (i := LC.Start(aValue)) < 0                      ' get center, see if invalid
      mode~                                             ' if so, clear mode
      lc.stop                                           ' stop LC object  
  SetClock                                              ' start clock running.
Return i

PUB SetMax(aValue)                         ' Procedure - Set max value for slider
  if (SMax := aValue) > SMin
    Stopped := False
    if (mode & 1) > 0
      Gain := 1 #> (LCMax / (SMax-SMin))
    else
      Gain := 1 #> (RCMax / (SMax-SMin))
  else
    Stopped := True

PUB SetMin(aValue)                         ' Procedure - Set min value for slider
  if (SMin := aValue) < SMax
    Stopped := False
    if (mode & 1) > 0
      Gain := 1 #> (LCMax / (SMax-SMin))
    else
      Gain := 1 #> (RCMax / (SMax-SMin))
  else
    Stopped := True

PUB Data(Number)                           ' redirect data to leds
  Led(Number)

PUB Msg(Ptr)

PUB User2
Return Ina[User2Pin] == 0                 ' Function - Boolean 

PUB Slider1 | Pot                           ' Function - Return the current pot value
  if (mode & 1) > 0 
    Pot := LC.GetAvg - LC.GetLeast
    pot := (pot >> Ignr_bit-3) << Ignr_bit-3     ' waste some bits.
  else
    rc.rctime(RCPin,1,@Pot)              ' read slider pot using rc.rctime method
    pot := (pot >> Ignr_bit) << Ignr_bit     ' waste some bits.
  if not stopped
    return Smin #> (Pot/Gain + Smin) <# SMax
  else
    return Pot

PUB SetSlider1(aValue)

Return

PUB DownLoad
  return false

PUB Beep(msec)
  if (mode & 4) > 0 
    leds.beep(msec)

PUB Pause(msec)
  if (mode & 4) > 0 
    leds.pause(msec)
     
PUB Label(Ptr)
  Return

PUB HHMMSS | i
  if (mode & 2) > 0                                 ' if clock used, then ...
    rtc.readTime( @hour, @minute, @second )
    i := hour * 10_000 + minute * 100 + second
  else
    i~
return i

PUB LED(aValue)
  if (mode & 4) > 0 
    leds.SetValue(aValue)

PUB SetClock | cmd                '' called once at startup.

    ' important note: real time clock (DS1302) and the thermometer (DS1620) share
    ' the same clock and data lines!
  if (mode & 2) > 0
    rtc.init( 21, 20, 19 )                          ' ports (Clock, data, chip enable)
    rtc.config                                      ' Set configuration register
    rtc.tcOFF                                       ' turn off trickle charger.
     